VERSION 5.00
Begin VB.Form Form1 
   Caption         =   "Mandelbrot"
   ClientHeight    =   4155
   ClientLeft      =   165
   ClientTop       =   855
   ClientWidth     =   4485
   LinkTopic       =   "Form1"
   ScaleHeight     =   4155
   ScaleWidth      =   4485
   StartUpPosition =   3  'Windows Default
   Begin VB.PictureBox picCanvas 
      Height          =   3735
      Left            =   120
      ScaleHeight     =   3675
      ScaleWidth      =   3915
      TabIndex        =   0
      Top             =   120
      Width           =   3975
   End
   Begin VB.Menu mnuFile 
      Caption         =   "&File"
      Begin VB.Menu mnuFileSaveAs 
         Caption         =   "Save &As..."
      End
   End
   Begin VB.Menu mnuScale 
      Caption         =   "&Scale"
      Begin VB.Menu mnuScaleFactor 
         Caption         =   "x&2"
         Index           =   2
      End
      Begin VB.Menu mnuScaleFactor 
         Caption         =   "x&4"
         Index           =   4
      End
      Begin VB.Menu mnuScaleFactor 
         Caption         =   "x&8"
         Index           =   8
      End
      Begin VB.Menu mnuScaleFull 
         Caption         =   "&Full"
      End
      Begin VB.Menu mnuScaleSep 
         Caption         =   "-"
      End
      Begin VB.Menu mnuScaleRefresh 
         Caption         =   "&Refresh"
         Shortcut        =   {F5}
      End
   End
   Begin VB.Menu mnuColors 
      Caption         =   "&Colors"
      Begin VB.Menu mnuColorsOriginal 
         Caption         =   "&Original"
         Checked         =   -1  'True
      End
      Begin VB.Menu mnuColorsSmooth1 
         Caption         =   "Smooth &1"
      End
      Begin VB.Menu mnuColorsSmooth2 
         Caption         =   "Smooth &2"
      End
   End
End
Attribute VB_Name = "Form1"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
Option Explicit

Private m_DrawingBox As Boolean
Private m_StartX As Single
Private m_StartY As Single
Private m_CurX As Single
Private m_CurY As Single
Private m_Xmin As Double
Private m_Xmax As Double
Private m_Ymin As Double
Private m_Ymax As Double

Public MaxIterations As Integer
Public Z0 As Complex

Private Const MIN_X As Double = -2.2
Private Const MAX_X As Double = 1
Private Const MIN_Y As Double = -1.2
Private Const MAX_Y As Double = 1.2

Private Colors() As OLE_COLOR
Private NumColors As Integer

Private Sub Form_Load()
    Me.Show
    DoEvents

    picCanvas.AutoRedraw = True
    picCanvas.ScaleMode = vbPixels

    MaxIterations = 64
    Set Z0 = New Complex

    NumColors = 8
    ReDim Colors(0 To NumColors - 1)
    Colors(0) = RGB(255, 0, 0)
    Colors(1) = RGB(255, 128, 0)
    Colors(2) = RGB(255, 255, 0)
    Colors(3) = RGB(0, 192, 0)
    Colors(4) = RGB(0, 255, 255)
    Colors(5) = RGB(0, 0, 255)
    Colors(6) = RGB(255, 0, 255)
    Colors(7) = RGB(0, 0, 0)

    ' Display the first Mandelbrot set.
    mnuScaleFull_Click
End Sub

Private Sub mnuColorsOriginal_Click()
    mnuColorsOriginal.Checked = True
    mnuColorsSmooth1.Checked = False
    mnuColorsSmooth2.Checked = False
End Sub
Private Sub mnuColorsSmooth1_Click()
    mnuColorsOriginal.Checked = False
    mnuColorsSmooth1.Checked = True
    mnuColorsSmooth2.Checked = False
End Sub
Private Sub mnuColorsSmooth2_Click()
    mnuColorsOriginal.Checked = False
    mnuColorsSmooth1.Checked = False
    mnuColorsSmooth2.Checked = True
End Sub

Private Sub picCanvas_MouseDown(Button As Integer, Shift As Integer, X As Single, Y As Single)
    m_DrawingBox = True
    m_StartX = X
    m_StartY = Y
    m_CurX = X
    m_CurY = Y
End Sub

Private Sub picCanvas_MouseMove(Button As Integer, Shift As Integer, X As Single, Y As Single)
    If Not m_DrawingBox Then Exit Sub

    m_CurX = X
    m_CurY = Y
    picCanvas.Cls
    picCanvas.Line (m_StartX, m_StartY)-(m_CurX, m_CurY), vbYellow, B
End Sub

Private Sub picCanvas_MouseUp(Button As Integer, Shift As Integer, X As Single, Y As Single)
Dim x1 As Double
Dim x2 As Double
Dim y1 As Double
Dim y2 As Double
Dim factor As Double
    
    If Not m_DrawingBox Then Exit Sub
    m_DrawingBox = False
    picCanvas.Cls

    m_CurX = X
    m_CurY = Y

    ' Put the coordinates in proper order.
    x1 = Min(m_StartX, m_CurX)
    x2 = Max(m_StartX, m_CurX)
    If x1 = x2 Then x2 = x1 + 1

    y1 = Min(m_StartY, m_CurY)
    y2 = Max(m_StartY, m_CurY)
    If y1 = y2 Then y2 = y1 + 1

    ' Convert screen coords into drawing coords.
    factor = (m_Xmax - m_Xmin) / picCanvas.ScaleWidth
    m_Xmax = m_Xmin + x2 * factor
    m_Xmin = m_Xmin + x1 * factor

    factor = (m_Ymax - m_Ymin) / picCanvas.ScaleHeight
    m_Ymax = m_Ymin + y2 * factor
    m_Ymin = m_Ymin + y1 * factor

    Me.MousePointer = vbHourglass
    DoEvents
    DrawMandelbrot
    Me.MousePointer = vbDefault
    picCanvas.MousePointer = vbCrosshair
End Sub

Private Sub Form_Resize()
    picCanvas.Move 0, 0, ScaleWidth, ScaleHeight
End Sub

Private Sub mnuFileSaveAs_Click()
    MsgBox "Not yet implemented."
End Sub

Private Sub mnuScaleFactor_Click(Index As Integer)
    ScaleArea Index
End Sub

' Adjust the aspect ratio of the selected
' coordinates so they fit the window properly.
Private Sub AdjustAspect()
Dim want_aspect As Double
Dim picCanvas_aspect As Double
Dim hgt As Double
Dim wid As Double
Dim mid As Double

    want_aspect = (m_Ymax - m_Ymin) / (m_Xmax - m_Xmin)
    picCanvas_aspect = picCanvas.ScaleHeight / picCanvas.ScaleWidth
    If want_aspect > picCanvas_aspect Then
        ' The selected area is too tall and thin.
        ' Make it wider.
        wid = (m_Ymax - m_Ymin) / picCanvas_aspect
        mid = (m_Xmin + m_Xmax) / 2
        m_Xmin = mid - wid / 2
        m_Xmax = mid + wid / 2
    Else
        ' The selected area is too short and wide.
        ' Make it taller.
        hgt = (m_Xmax - m_Xmin) * picCanvas_aspect
        mid = (m_Ymin + m_Ymax) / 2
        m_Ymin = mid - hgt / 2
        m_Ymax = mid + hgt / 2
    End If
End Sub

' Draw the Mandelbrot set.
Private Sub DrawMandelbrot()
' Work until |Zn| >= escape_radius
Const escape_radius As Double = 2#

Dim wid As Integer
Dim hgt As Integer
Dim iteration As Integer
Dim X As Integer
Dim Y As Integer
Dim Z As Complex
Dim C As Complex
Dim ReaC As Double
Dim ImaC As Double
Dim dReaC As Double
Dim dImaC As Double
Dim escape_squared As Double
Dim log_escape As Double
Dim i As Integer
Dim mu As Double

    ' Work until the |Zn|^2 >= escape_radius^2.
    escape_squared = escape_radius * escape_radius

    ' Used for color calculations.
    log_escape = Log(escape_radius)
    
    ' Clear.
    picCanvas.Line (0, 0)-(picCanvas.ScaleWidth, picCanvas.ScaleHeight), vbBlack, BF
    DoEvents

    ' Adjust the coordinate bounds to fit picCanvas.
    AdjustAspect

    ' dReaC is the change in the real part
    ' (X value) for C. dImaC is the change in the
    ' imaginary part (Y value).
    wid = picCanvas.ScaleWidth
    hgt = picCanvas.ScaleHeight
    dReaC = (m_Xmax - m_Xmin) / (wid - 1)
    dImaC = (m_Ymax - m_Ymin) / (hgt - 1)

    ' Calculate the values.
    ReaC = m_Xmin
    For X = 0 To wid - 1
        ImaC = m_Ymin
        For Y = 0 To hgt - 1
            Set Z = New Complex
            Z.Initialize Z0.Real, Z0.Imaginary
                    
            Set C = New Complex
            C.Initialize ReaC, ImaC
            
            iteration = 1
            Do While iteration < MaxIterations And Z.MagnitudeSquared() < escape_squared
                ' Calculate Z(iteration).
                Set Z = Z.Times(Z).Plus(C)
                iteration = iteration + 1
            Loop

            ' Set the pixel's value.
            If (iteration >= MaxIterations) Then
                picCanvas.PSet (X, Y), vbBlack
            Else
                If mnuColorsOriginal.Checked Then
                    picCanvas.PSet (X, Y), Colors(iteration Mod NumColors)
                Else
                    ' Reduce the error in mu.
                    For i = 0 To 2
                        Set Z = Z.Times(Z).Plus(C)
                        iteration = iteration + 1
                    Next i

                    mu = iteration + 1 - _
                        Log(Log(Z.Magnitude)) / log_escape
                    If mnuColorsSmooth2.Checked Then
                        mu = mu / MaxIterations * NumColors
                    End If

                    picCanvas.PSet (X, Y), GetColor(mu)
                End If
            End If

            ImaC = ImaC + dImaC
        Next Y
        ReaC = ReaC + dReaC

        ' Let the user know we're not dead.
        If X Mod 10 = 0 Then
            picCanvas.Refresh
        End If
    Next X

    Me.Caption = "Mandelbrot (" & _
        m_Xmin & ", " & m_Ymin & ")-(" & _
        m_Xmax & ", " & m_Ymax & ")"

    picCanvas.Picture = picCanvas.Image
End Sub

' Scale the selected area by this factor.
Private Sub ScaleArea(ByVal scale_factor As Integer)
Dim size As Double
Dim mid As Double

    size = scale_factor * (m_Xmax - m_Xmin)
    If size > 3.2 Then
        mnuScaleFull_Click
        Exit Sub
    End If
    mid = (m_Xmin + m_Xmax) / 2
    m_Xmin = mid - size / 2
    m_Xmax = mid + size / 2

    size = scale_factor * (m_Ymax - m_Ymin)
    If size > 2.4 Then
        mnuScaleFull_Click
        Exit Sub
    End If
    mid = (m_Ymin + m_Ymax) / 2
    m_Ymin = mid - size / 2
    m_Ymax = mid + size / 2

    Me.MousePointer = vbHourglass
    DoEvents
    DrawMandelbrot
    Me.MousePointer = vbDefault
    picCanvas.MousePointer = vbCrosshair
End Sub

' Zoom out to full scale.
Private Sub mnuScaleFull_Click()
    m_Xmin = MIN_X
    m_Xmax = MAX_X
    m_Ymin = MIN_Y
    m_Ymax = MAX_Y

    Me.MousePointer = vbHourglass
    DoEvents
    DrawMandelbrot
    Me.MousePointer = vbDefault
    picCanvas.MousePointer = vbCrosshair
End Sub

' Refresh. This is useful when the user resizes the form.
Private Sub mnuScaleRefresh_Click()
    ScaleArea 1
End Sub

Private Function Min(ByVal v1 As Double, ByVal v2 As Double) As Double
    If v1 < v2 Then
        Min = v1
    Else
        Min = v2
    End If
End Function
Private Function Max(ByVal v1 As Double, ByVal v2 As Double) As Double
    If v1 < v2 Then
        Max = v2
    Else
        Max = v1
    End If
End Function
' Get a color for this pixel.
Private Function GetColor(ByVal mu As Double) As OLE_COLOR
Dim clr1 As Integer
Dim clr2 As Integer
Dim t2 As Double
Dim t1 As Double
Dim r1 As Byte
Dim g1 As Byte
Dim b1 As Byte
Dim r2 As Byte
Dim g2 As Byte
Dim b2 As Byte
Dim r As Integer
Dim g As Integer
Dim b As Integer

    clr1 = CInt(Int(mu))
    t2 = mu - clr1
    t1 = 1 - t2
    clr1 = clr1 Mod NumColors
    clr2 = (clr1 + 1) Mod NumColors

    UnRGB Colors(clr1), r1, g1, b1
    UnRGB Colors(clr2), r2, g2, b2

    r = CInt(r1 * t1 + r2 * t2)
    g = CInt(g1 * t1 + g2 * t2)
    b = CInt(b1 * t1 + b2 * t2)

    GetColor = RGB(r, g, b)
End Function

' Return a color's red, green, and blue components.
Private Sub UnRGB(ByRef color As OLE_COLOR, ByRef r As Byte, ByRef g As Byte, ByRef b As Byte)
    r = color And &HFF&
    g = (color And &HFF00&) \ &H100&
    b = (color And &HFF0000) \ &H10000
End Sub

